@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -19,28 +19,34 @@
   3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #include "../linux/SDL_evdev_kbd.h"
-#include "SDL_hints.h"
 
 #ifdef SDL_INPUT_FBSDKBIO
 
-/* This logic is adapted from drivers/tty/vt/keyboard.c in the Linux kernel source, slightly modified to work with FreeBSD */
+// This logic is adapted from drivers/tty/vt/keyboard.c in the Linux kernel source, slightly modified to work with FreeBSD
 
+#include <errno.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <sys/kbio.h>
 #include <sys/consio.h>
+#include <termios.h>
 
 #include <signal.h>
 
 #include "../../events/SDL_events_c.h"
 #include "SDL_evdev_kbd_default_keyaccmap.h"
 
+#ifndef K_OFF
+#define K_OFF 0x04
+#endif
+
 typedef void(fn_handler_fn)(SDL_EVDEV_keyboard_state *kbd);
 
+
 /*
  * Keyboard State
  */
@@ -49,24 +55,29 @@ struct SDL_EVDEV_keyboard_state
 {
     int console_fd;
     int keyboard_fd;
+    bool muted;
     unsigned long old_kbd_mode;
     unsigned short **key_maps;
     keymap_t *key_map;
     keyboard_info_t *kbInfo;
-    unsigned char shift_down[4]; /* shift state counters.. */
-    SDL_bool dead_key_next;
-    int npadch; /* -1 or number assembled on pad */
+    unsigned char shift_down[4]; // shift state counters..
+    bool dead_key_next;
+    int npadch; // -1 or number assembled on pad
     accentmap_t *accents;
     unsigned int diacr;
-    SDL_bool rep; /* flag telling character repeat */
+    bool rep; // flag telling character repeat
     unsigned char lockstate;
     unsigned char ledflagstate;
     char shift_state;
     char text[128];
     unsigned int text_len;
+    void (*vt_release_callback)(void *);
+    void *vt_release_callback_data;
+    void (*vt_acquire_callback)(void *);
+    void *vt_acquire_callback_data;
 };
 
-static int SDL_EVDEV_kbd_load_keymaps(SDL_EVDEV_keyboard_state *kbd)
+static bool SDL_EVDEV_kbd_load_keymaps(SDL_EVDEV_keyboard_state *kbd)
 {
     return ioctl(kbd->keyboard_fd, GIO_KEYMAP, kbd->key_map) >= 0;
 }
@@ -78,28 +89,34 @@ static int kbd_cleanup_atexit_installed = 0;
 static struct sigaction old_sigaction[NSIG];
 
 static int fatal_signals[] = {
-    /* Handlers for SIGTERM and SIGINT are installed in SDL_QuitInit. */
+    // Handlers for SIGTERM and SIGINT are installed in SDL_InitQuit.
     SIGHUP, SIGQUIT, SIGILL, SIGABRT,
     SIGFPE, SIGSEGV, SIGPIPE, SIGBUS,
     SIGSYS
 };
 
-static void kbd_cleanup(void)
+static void vt_update_mouse(SDL_EVDEV_keyboard_state *kbd, int operation)
 {
     struct mouse_info mData;
+
+    SDL_zero(mData);
+    mData.operation = operation;
+    ioctl(kbd->console_fd, CONS_MOUSECTL, &mData);
+}
+
+static void kbd_cleanup(void)
+{
     SDL_EVDEV_keyboard_state *kbd = kbd_cleanup_state;
-    if (kbd == NULL) {
+    if (!kbd) {
         return;
     }
     kbd_cleanup_state = NULL;
-    SDL_zero(mData);
-    mData.operation = MOUSE_SHOW;
     ioctl(kbd->keyboard_fd, KDSKBMODE, kbd->old_kbd_mode);
     if (kbd->keyboard_fd != kbd->console_fd) {
         close(kbd->keyboard_fd);
     }
     ioctl(kbd->console_fd, CONS_SETKBD, (unsigned long)(kbd->kbInfo->kb_index));
-    ioctl(kbd->console_fd, CONS_MOUSECTL, &mData);
+    vt_update_mouse(kbd, true);
 }
 
 void SDL_EVDEV_kbd_reraise_signal(int sig)
@@ -115,28 +132,28 @@ static void kbd_cleanup_signal_action(int signum, siginfo_t *info, void *ucontex
     struct sigaction *old_action_p = &(old_sigaction[signum]);
     sigset_t sigset;
 
-    /* Restore original signal handler before going any further. */
+    // Restore original signal handler before going any further.
     sigaction(signum, old_action_p, NULL);
 
-    /* Unmask current signal. */
+    // Unmask current signal.
     sigemptyset(&sigset);
     sigaddset(&sigset, signum);
     sigprocmask(SIG_UNBLOCK, &sigset, NULL);
 
-    /* Save original signal info and context for archeologists. */
+    // Save original signal info and context for archeologists.
     SDL_EVDEV_kdb_cleanup_siginfo = info;
     SDL_EVDEV_kdb_cleanup_ucontext = ucontext;
 
-    /* Restore keyboard. */
+    // Restore keyboard.
     kbd_cleanup();
 
-    /* Reraise signal. */
+    // Reraise signal.
     SDL_EVDEV_kbd_reraise_signal(signum);
 }
 
-static void kbd_unregister_emerg_cleanup()
+static void kbd_unregister_emerg_cleanup(void)
 {
-    int tabidx, signum;
+    int tabidx;
 
     kbd_cleanup_state = NULL;
 
@@ -145,41 +162,41 @@ static void kbd_unregister_emerg_cleanup()
     }
     kbd_cleanup_sigactions_installed = 0;
 
-    for (tabidx = 0; tabidx < sizeof(fatal_signals) / sizeof(fatal_signals[0]); ++tabidx) {
+    for (tabidx = 0; tabidx < SDL_arraysize(fatal_signals); ++tabidx) {
         struct sigaction *old_action_p;
         struct sigaction cur_action;
-        signum = fatal_signals[tabidx];
+        int signum = fatal_signals[tabidx];
         old_action_p = &(old_sigaction[signum]);
 
-        /* Examine current signal action */
+        // Examine current signal action
         if (sigaction(signum, NULL, &cur_action)) {
             continue;
         }
 
-        /* Check if action installed and not modifed */
+        // Check if action installed and not modified
         if (!(cur_action.sa_flags & SA_SIGINFO) || cur_action.sa_sigaction != &kbd_cleanup_signal_action) {
             continue;
         }
 
-        /* Restore original action */
+        // Restore original action
         sigaction(signum, old_action_p, NULL);
     }
 }
 
 static void kbd_cleanup_atexit(void)
 {
-    /* Restore keyboard. */
+    // Restore keyboard.
     kbd_cleanup();
 
-    /* Try to restore signal handlers in case shared library is being unloaded */
+    // Try to restore signal handlers in case shared library is being unloaded
     kbd_unregister_emerg_cleanup();
 }
 
 static void kbd_register_emerg_cleanup(SDL_EVDEV_keyboard_state *kbd)
 {
-    int tabidx, signum;
+    int tabidx;
 
-    if (kbd_cleanup_state != NULL) {
+    if (kbd_cleanup_state) {
         return;
     }
     kbd_cleanup_state = kbd;
@@ -198,10 +215,10 @@ static void kbd_register_emerg_cleanup(SDL_EVDEV_keyboard_state *kbd)
     }
     kbd_cleanup_sigactions_installed = 1;
 
-    for (tabidx = 0; tabidx < sizeof(fatal_signals) / sizeof(fatal_signals[0]); ++tabidx) {
+    for (tabidx = 0; tabidx < SDL_arraysize(fatal_signals); ++tabidx) {
         struct sigaction *old_action_p;
         struct sigaction new_action;
-        signum = fatal_signals[tabidx];
+        int signum = fatal_signals[tabidx];
         old_action_p = &(old_sigaction[signum]);
         if (sigaction(signum, NULL, old_action_p)) {
             continue;
@@ -221,33 +238,125 @@ static void kbd_register_emerg_cleanup(SDL_EVDEV_keyboard_state *kbd)
     }
 }
 
+enum {
+    VT_SIGNAL_NONE,
+    VT_SIGNAL_RELEASE,
+    VT_SIGNAL_ACQUIRE,
+};
+static int vt_release_signal;
+static int vt_acquire_signal;
+static SDL_AtomicInt vt_signal_pending;
+SDL_AtomicInt vt_current;
+
+typedef void (*signal_handler)(int signum);
+
+
+static void kbd_vt_release_signal_action(int signum)
+{
+    SDL_SetAtomicInt(&vt_signal_pending, VT_SIGNAL_RELEASE);
+    SDL_SetAtomicInt(&vt_current, VT_THEIRS);
+}
+
+static void kbd_vt_acquire_signal_action(int signum)
+{
+    SDL_SetAtomicInt(&vt_signal_pending, VT_SIGNAL_ACQUIRE);
+    SDL_SetAtomicInt(&vt_current, VT_OURS);
+}
+
+static bool setup_vt_signal(int signum, signal_handler handler)
+{
+    struct sigaction *old_action_p;
+    struct sigaction new_action;
+    old_action_p = &(old_sigaction[signum]);
+    SDL_zero(new_action);
+    new_action.sa_handler = handler;
+    new_action.sa_flags = SA_RESTART;
+    if (sigaction(signum, &new_action, old_action_p) < 0) {
+        return false;
+    }
+    if (old_action_p->sa_handler != SIG_DFL) {
+        // This signal is already in use
+        if (signum == SIGUSR1 || signum == SIGUSR2) {
+            /*
+             * For the moment we have no other options in FreeBSD because
+             * the vt(4) will not accept signal numbers over 32.
+             */
+            return true;
+        }
+        sigaction(signum, old_action_p, NULL);
+        return false;
+    }
+    return true;
+}
+
+static void kbd_vt_quit(int console_fd)
+{
+    struct vt_mode mode;
+
+    if (vt_release_signal) {
+        sigaction(vt_release_signal, &old_sigaction[vt_release_signal], NULL);
+        vt_release_signal = 0;
+    }
+    if (vt_acquire_signal) {
+        sigaction(vt_acquire_signal, &old_sigaction[vt_acquire_signal], NULL);
+        vt_acquire_signal = 0;
+    }
+
+    SDL_zero(mode);
+    mode.mode = VT_AUTO;
+    ioctl(console_fd, VT_SETMODE, &mode);
+}
+
+static bool kbd_vt_init(int console_fd)
+{
+    struct vt_mode mode;
+
+    if (setup_vt_signal(SIGUSR1, kbd_vt_release_signal_action)) {
+        vt_release_signal = SIGUSR1;
+    }
+    if (setup_vt_signal(SIGUSR2, kbd_vt_acquire_signal_action)) {
+        vt_acquire_signal = SIGUSR2;
+    }
+    if (!vt_release_signal || !vt_acquire_signal ) {
+        kbd_vt_quit(console_fd);
+        return false;
+    }
+
+    SDL_zero(mode);
+    mode.mode = VT_PROCESS;
+    mode.relsig = vt_release_signal;
+    mode.acqsig = vt_acquire_signal;
+    mode.frsig = SIGIO;
+    if (ioctl(console_fd, VT_SETMODE, &mode) < 0) {
+        SDL_LogError(SDL_LOG_CATEGORY_INPUT, "Failed VT_SETMODE ioctl: %s", strerror(errno));
+        kbd_vt_quit(console_fd);
+        return false;
+    }
+    return true;
+}
+
 SDL_EVDEV_keyboard_state *SDL_EVDEV_kbd_init(void)
 {
     SDL_EVDEV_keyboard_state *kbd;
-    struct mouse_info mData;
     char flag_state;
-    char *devicePath;
 
-    SDL_zero(mData);
-    mData.operation = MOUSE_HIDE;
     kbd = (SDL_EVDEV_keyboard_state *)SDL_calloc(1, sizeof(SDL_EVDEV_keyboard_state));
-    if (kbd == NULL) {
+    if (!kbd) {
         return NULL;
     }
 
     kbd->npadch = -1;
 
-    /* This might fail if we're not connected to a tty (e.g. on the Steam Link) */
+    // This might fail if we're not connected to a tty (e.g. on the Steam Link)
     kbd->keyboard_fd = kbd->console_fd = open("/dev/tty", O_RDONLY | O_CLOEXEC);
 
     kbd->shift_state = 0;
 
-    kbd->accents = SDL_calloc(sizeof(accentmap_t), 1);
-    kbd->key_map = SDL_calloc(sizeof(keymap_t), 1);
-    kbd->kbInfo = SDL_calloc(sizeof(keyboard_info_t), 1);
+    kbd->accents = SDL_calloc(1, sizeof(accentmap_t));
+    kbd->key_map = SDL_calloc(1, sizeof(keymap_t));
+    kbd->kbInfo = SDL_calloc(1, sizeof(keyboard_info_t));
 
     ioctl(kbd->console_fd, KDGKBINFO, kbd->kbInfo);
-    ioctl(kbd->console_fd, CONS_MOUSECTL, &mData);
 
     if (ioctl(kbd->console_fd, KDGKBSTATE, &flag_state) == 0) {
         kbd->ledflagstate = flag_state;
@@ -259,55 +368,43 @@ SDL_EVDEV_keyboard_state *SDL_EVDEV_kbd_init(void)
     }
 
     if (ioctl(kbd->console_fd, KDGKBMODE, &kbd->old_kbd_mode) == 0) {
-        /* Set the keyboard in XLATE mode and load the keymaps */
+        // Set the keyboard in XLATE mode and load the keymaps
         ioctl(kbd->console_fd, KDSKBMODE, (unsigned long)(K_XLATE));
         if (!SDL_EVDEV_kbd_load_keymaps(kbd)) {
             SDL_free(kbd->key_map);
             kbd->key_map = &keymap_default_us_acc;
         }
-        /* Allow inhibiting keyboard mute with env. variable for debugging etc. */
-        if (SDL_getenv("SDL_INPUT_FREEBSD_KEEP_KBD") == NULL) {
-            /* Take keyboard from console and open the actual keyboard device.
-             * Ensures that the keystrokes do not leak through to the console.
-             */
-            ioctl(kbd->console_fd, CONS_RELKBD, 1ul);
-            SDL_asprintf(&devicePath, "/dev/kbd%d", kbd->kbInfo->kb_index);
-            kbd->keyboard_fd = open(devicePath, O_WRONLY | O_CLOEXEC);
-            if (kbd->keyboard_fd == -1) {
-                // Give keyboard back.
-                ioctl(kbd->console_fd, CONS_SETKBD, (unsigned long)(kbd->kbInfo->kb_index));
-                kbd->keyboard_fd = kbd->console_fd;
-            }
 
-            /* Make sure to restore keyboard if application fails to call
-             * SDL_Quit before exit or fatal signal is raised.
-             */
-            if (!SDL_GetHintBoolean(SDL_HINT_NO_SIGNAL_HANDLERS, SDL_FALSE)) {
-                kbd_register_emerg_cleanup(kbd);
-            }
-            SDL_free(devicePath);
-        } else
-            kbd->keyboard_fd = kbd->console_fd;
+        if (!kbd_vt_init(kbd->console_fd)) {
+            SDL_LogInfo(SDL_LOG_CATEGORY_INPUT, "kbd_vt_init failed");
+        }
+
+        kbd->keyboard_fd = kbd->console_fd;
+
+        if (!SDL_GetHintBoolean(SDL_HINT_NO_SIGNAL_HANDLERS, false)) {
+            kbd_register_emerg_cleanup(kbd);
+        }
     }
 
+    vt_update_mouse(kbd, MOUSE_HIDE);
+
     return kbd;
 }
 
 void SDL_EVDEV_kbd_quit(SDL_EVDEV_keyboard_state *kbd)
 {
-    struct mouse_info mData;
-
-    if (kbd == NULL) {
+    if (!kbd) {
         return;
     }
-    SDL_zero(mData);
-    mData.operation = MOUSE_SHOW;
-    ioctl(kbd->console_fd, CONS_MOUSECTL, &mData);
+
+    kbd_vt_quit(kbd->console_fd);
+
+    vt_update_mouse(kbd, MOUSE_SHOW);
 
     kbd_unregister_emerg_cleanup();
 
     if (kbd->keyboard_fd >= 0) {
-        /* Restore the original keyboard mode */
+        // Restore the original keyboard mode
         ioctl(kbd->keyboard_fd, KDSKBMODE, kbd->old_kbd_mode);
 
         close(kbd->keyboard_fd);
@@ -321,12 +418,93 @@ void SDL_EVDEV_kbd_quit(SDL_EVDEV_keyboard_state *kbd)
     SDL_free(kbd);
 }
 
+void SDL_EVDEV_kbd_set_muted(SDL_EVDEV_keyboard_state *state, bool muted)
+{
+    struct termios tios;
+
+    SDL_zero(tios);
+
+    if (!state) {
+        return;
+    }
+
+    if (muted == state->muted) {
+        return;
+    }
+
+    if (tcgetattr(state->console_fd, &tios) == -1) {
+        SDL_LogError(SDL_LOG_CATEGORY_INPUT, "Could not get terminal mode: %s", strerror(errno));
+        return;
+    }
+
+    if (muted) {
+        if (SDL_GetHintBoolean(SDL_HINT_MUTE_CONSOLE_KEYBOARD, true)) {
+            ioctl(state->console_fd, KDSKBMODE, K_OFF);
+            cfmakeraw(&tios);
+
+            if (!SDL_GetHintBoolean(SDL_HINT_NO_SIGNAL_HANDLERS, false)) {
+                kbd_register_emerg_cleanup(state);
+            }
+        }
+    } else {
+        kbd_unregister_emerg_cleanup();
+
+        cfmakesane(&tios);
+        ioctl(state->console_fd, KDSKBMODE, state->old_kbd_mode);
+    }
+
+    if (tcsetattr(state->console_fd, TCSAFLUSH, &tios) == -1) {
+        SDL_LogError(SDL_LOG_CATEGORY_INPUT, "Could not set terminal mode to %s: %s", muted ? "raw" : "sane", strerror(errno));
+        return;
+    }
+
+    state->muted = muted;
+}
+
+void SDL_EVDEV_kbd_set_vt_switch_callbacks(SDL_EVDEV_keyboard_state *state, void (*release_callback)(void *), void *release_callback_data, void (*acquire_callback)(void *), void *acquire_callback_data)
+{
+    if (state == NULL) {
+        return;
+    }
+
+    state->vt_release_callback = release_callback;
+    state->vt_release_callback_data = release_callback_data;
+    state->vt_acquire_callback = acquire_callback;
+    state->vt_acquire_callback_data = acquire_callback_data;
+}
+
+void SDL_EVDEV_kbd_update(SDL_EVDEV_keyboard_state *state)
+{
+    if (!state) {
+        return;
+    }
+
+    int signal_pending = SDL_GetAtomicInt(&vt_signal_pending);
+
+    if (signal_pending != VT_SIGNAL_NONE) {
+        if (signal_pending == VT_SIGNAL_RELEASE) {
+            if (state->vt_release_callback) {
+                vt_update_mouse(state, MOUSE_SHOW);
+                state->vt_release_callback(state->vt_release_callback_data);
+            }
+            ioctl(state->console_fd, VT_RELDISP, 1);
+        } else {
+            if (state->vt_acquire_callback) {
+                state->vt_acquire_callback(state->vt_acquire_callback_data);
+                vt_update_mouse(state, MOUSE_HIDE);
+            }
+            ioctl(state->console_fd, VT_RELDISP, VT_ACKACQ);
+        }
+        SDL_CompareAndSwapAtomicInt(&vt_signal_pending, signal_pending, VT_SIGNAL_NONE);
+    }
+}
+
 /*
  * Helper Functions.
  */
 static void put_queue(SDL_EVDEV_keyboard_state *kbd, uint c)
 {
-    /* c is already part of a UTF-8 sequence and safe to add as a character */
+    // c is already part of a UTF-8 sequence and safe to add as a character
     if (kbd->text_len < (sizeof(kbd->text) - 1)) {
         kbd->text[kbd->text_len++] = (char)c;
     }
@@ -378,7 +556,7 @@ static unsigned int handle_diacr(SDL_EVDEV_keyboard_state *kbd, unsigned int ch)
     for (i = 0; i < kbd->accents->n_accs; i++) {
         if (kbd->accents->acc[i].accchar == d) {
             for (j = 0; j < NUM_ACCENTCHARS; ++j) {
-                if (kbd->accents->acc[i].map[j][0] == 0) { /* end of table */
+                if (kbd->accents->acc[i].map[j][0] == 0) { // end of table
                     break;
                 }
                 if (kbd->accents->acc[i].map[j][0] == ch) {
@@ -397,7 +575,7 @@ static unsigned int handle_diacr(SDL_EVDEV_keyboard_state *kbd, unsigned int ch)
     return ch;
 }
 
-static int vc_kbd_led(SDL_EVDEV_keyboard_state *kbd, int flag)
+static bool vc_kbd_led(SDL_EVDEV_keyboard_state *kbd, int flag)
 {
     return (kbd->ledflagstate & flag) != 0;
 }
@@ -415,7 +593,7 @@ static void chg_vc_kbd_led(SDL_EVDEV_keyboard_state *kbd, int flag)
 static void k_self(SDL_EVDEV_keyboard_state *kbd, unsigned int value, char up_flag)
 {
     if (up_flag) {
-        return; /* no action, if this is a key release */
+        return; // no action, if this is a key release
     }
 
     if (kbd->diacr) {
@@ -423,7 +601,7 @@ static void k_self(SDL_EVDEV_keyboard_state *kbd, unsigned int value, char up_fl
     }
 
     if (kbd->dead_key_next) {
-        kbd->dead_key_next = SDL_FALSE;
+        kbd->dead_key_next = false;
         kbd->diacr = value;
         return;
     }
@@ -461,7 +639,7 @@ static void k_shift(SDL_EVDEV_keyboard_state *kbd, unsigned char value, char up_
     else
         kbd->shift_state &= ~(1 << value);
 
-    /* kludge */
+    // kludge
     if (up_flag && kbd->shift_state != old_state && kbd->npadch != -1) {
         put_utf8(kbd, kbd->npadch);
         kbd->npadch = -1;
@@ -475,7 +653,7 @@ void SDL_EVDEV_kbd_keycode(SDL_EVDEV_keyboard_state *kbd, unsigned int keycode,
     unsigned int final_key_state;
     unsigned int map_from_key_sym;
 
-    if (kbd == NULL) {
+    if (!kbd) {
         return;
     }
 
@@ -485,7 +663,7 @@ void SDL_EVDEV_kbd_keycode(SDL_EVDEV_keyboard_state *kbd, unsigned int keycode,
 
     if (keycode < NUM_KEYS) {
         if (keycode >= 89 && keycode <= 95) {
-            /* These constitute unprintable language-related keys, so ignore them. */
+            // These constitute unprintable language-related keys, so ignore them.
             return;
         }
         if (keycode > 95) {
@@ -509,66 +687,69 @@ void SDL_EVDEV_kbd_keycode(SDL_EVDEV_keyboard_state *kbd, unsigned int keycode,
 
     map_from_key_sym = keysym.map[final_key_state];
     if ((keysym.spcl & (0x80 >> final_key_state)) || (map_from_key_sym & SPCLKEY)) {
-        /* Special function.*/
+        // Special function.
         if (map_from_key_sym == 0)
-            return; /* Nothing to do. */
+            return; // Nothing to do.
         if (map_from_key_sym & SPCLKEY) {
             map_from_key_sym &= ~SPCLKEY;
         }
         if (map_from_key_sym >= F_ACC && map_from_key_sym <= L_ACC) {
-            /* Accent function.*/
+            // Accent function.
             unsigned int accent_index = map_from_key_sym - F_ACC;
             if (kbd->accents->acc[accent_index].accchar != 0) {
                 k_deadunicode(kbd, kbd->accents->acc[accent_index].accchar, !down);
             }
         } else {
             switch (map_from_key_sym) {
-            case ASH: /* alt/meta shift */
+            case ASH: // alt/meta shift
                 k_shift(kbd, 3, down == 0);
                 break;
-            case LSHA: /* left shift + alt lock */
-            case RSHA: /* right shift + alt lock */
+            case LSHA: // left shift + alt lock
+            case RSHA: // right shift + alt lock
                 if (down == 0) {
                     chg_vc_kbd_led(kbd, ALKED);
                 }
-            case LSH: /* left shift */
-            case RSH: /* right shift */
+                SDL_FALLTHROUGH;
+            case LSH: // left shift
+            case RSH: // right shift
                 k_shift(kbd, 0, down == 0);
                 break;
-            case LCTRA: /* left ctrl + alt lock */
-            case RCTRA: /* right ctrl + alt lock */
+            case LCTRA: // left ctrl + alt lock
+            case RCTRA: // right ctrl + alt lock
                 if (down == 0) {
                     chg_vc_kbd_led(kbd, ALKED);
                 }
-            case LCTR: /* left ctrl */
-            case RCTR: /* right ctrl */
+                SDL_FALLTHROUGH;
+            case LCTR: // left ctrl
+            case RCTR: // right ctrl
                 k_shift(kbd, 1, down == 0);
                 break;
-            case LALTA: /* left alt + alt lock */
-            case RALTA: /* right alt + alt lock */
+            case LALTA: // left alt + alt lock
+            case RALTA: // right alt + alt lock
                 if (down == 0) {
                     chg_vc_kbd_led(kbd, ALKED);
                 }
-            case LALT: /* left alt */
-            case RALT: /* right alt */
+                SDL_FALLTHROUGH;
+            case LALT: // left alt
+            case RALT: // right alt
                 k_shift(kbd, 2, down == 0);
                 break;
-            case ALK: /* alt lock */
+            case ALK: // alt lock
                 if (down == 1) {
                     chg_vc_kbd_led(kbd, ALKED);
                 }
                 break;
-            case CLK: /* caps lock*/
+            case CLK: // caps lock
                 if (down == 1) {
                     chg_vc_kbd_led(kbd, CLKED);
                 }
                 break;
-            case NLK: /* num lock */
+            case NLK: // num lock
                 if (down == 1) {
                     chg_vc_kbd_led(kbd, NLKED);
                 }
                 break;
-            case SLK: /* scroll lock */
+            case SLK: // scroll lock
                 if (down == 1) {
                     chg_vc_kbd_led(kbd, SLKED);
                 }
@@ -596,6 +777,4 @@ void SDL_EVDEV_kbd_keycode(SDL_EVDEV_keyboard_state *kbd, unsigned int keycode,
     }
 }
 
-#endif /* SDL_INPUT_FBSDKBIO */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_INPUT_FBSDKBIO
